@@ -3,7 +3,7 @@
 ;; Copyright (C) 2024  Free Software Foundation, Inc.
 
 ;; Author: Paul D. Nelson <nelson.paul.david@gmail.com>
-;; Version: 0.3
+;; Version: 0.4
 ;; URL: https://github.com/ultronozm/preview-auto.el
 ;; Package-Requires: ((emacs "29.3") (auctex "14.0.5"))
 ;; Keywords: tex, convenience
@@ -39,6 +39,7 @@
 
 ;;; Code:
 
+(eval-when-compile (require 'cl-lib))
 (require 'latex)
 (require 'preview)
 
@@ -73,8 +74,8 @@ For this to have any effect, it must be set before
   "List of modes in which to use font-lock to detect math environments.
 Applies also to any derived modes.  This is faster than using
 `texmathp', but may not work outside tex major modes without additional
-setup.  Set to `nil' to always use `texmathp' (e.g., if you work in
-LaTeX-mode with font-lock disabled)."
+setup.  Set to nil to always use `texmathp' (e.g., if you work in
+`LaTeX-mode' with font-lock disabled)."
   :type '(repeat symbol))
 
 (defun preview-auto--math-p ()
@@ -90,7 +91,6 @@ Should work in AUCTeX `LaTeX-mode' buffers.  Implemented using
                  (memq math-face face))))
     (texmathp)))
 
-
 (defun preview-auto--generate-rules ()
   "Return list of rules for identifying math environments."
   (let* ((basic-rules
@@ -120,7 +120,6 @@ is valid.")
 (defvar-local preview-auto--begin-re nil
   "Regular expression for identifying the beginning of a math environment.")
 
-
 (defun preview-auto--check-default ()
   "Default predicate for checking whether to consider a delimiter.
 Returns non-nil if either
@@ -151,9 +150,9 @@ returns nil."
 
 (defcustom preview-auto-refresh-after-compilation t
   "If non-nil, refresh previews after each compilation.
-This plays well with the packages `tex-numbers' and `tex-continuous':
-the result is that preview equation numbers are updated automatically to
-the correct form."
+This plays well with the packages `auctex-label-numbers' and
+`auctex-cont-latexmk': the result is that preview equation numbers are
+updated automatically to the correct form."
   :type 'boolean)
 
 (defconst preview-auto--refresh-delay '(0 1)
@@ -174,8 +173,8 @@ A preview is considered non-disabled if it is active or inactive
 according to `preview.el'.  If `preview-auto-refresh-after-compilation'
 is non-nil, then we further require that the preview has been generated
 more recently than the aux file.  This last requirement, when combined
-with the `tex-numbers' and `tex-continuous' packages, ensures that
-preview equation numbers are updated automatically."
+with the `auctex-label-numbers' and `auctex-cont-latexmk' packages,
+ensures that preview equation numbers are updated automatically."
   (seq-some
    (lambda (ov)
      (and
@@ -388,8 +387,10 @@ ORIG-FUN is the original function, and ARGS are its arguments."
                  (end (save-excursion
                         (preview-auto--search (regexp-quote end-string)
                                               limit))))
-              ;; Don't preview empty regions.
-              (when (string-match-p "[^[:space:]\n\r]"
+              ;; Don't preview essentially empty regions.  Exclude
+              ;; backslashes to avoid issues with previewing
+              ;; regions-in-progress such as "$\$".
+              (when (string-match-p "[^[:space:]\n\r\\]"
                                     (buffer-substring-no-properties
                                      (+ begin (length why))
                                      (- end (length end-string))))
@@ -520,29 +521,43 @@ cancel the preview, so that the preview is not misplaced."
   :lighter nil
   (cond
    (preview-auto-mode
-    (progn
-      (unless TeX-header-end
-        (setq TeX-header-end LaTeX-header-end))
-      (unless TeX-trailer-start
-        (setq TeX-trailer-start LaTeX-trailer-start))
-      (unless (memq #'preview-move-point post-command-hook)
-        (add-hook 'post-command-hook #'preview-move-point nil t))
-      (add-hook 'after-change-functions #'preview-auto--after-change nil t)
-      (add-hook 'post-command-hook #'preview-auto--post-command nil t)
-      (when preview-auto--timer
-        ;; Reset the timer, in case it's broken.
-        (cancel-timer preview-auto--timer)
-        (setq preview-auto--timer nil))
-      (setq preview-auto--timer (run-with-timer preview-auto-interval
-                                                preview-auto-interval
-                                                #'preview-auto--timer-function))
-      (setq preview-auto--keepalive t)))
+    (unless (or (derived-mode-p 'LaTeX-mode)
+                (and
+                 (stringp TeX-master)
+                 (file-exists-p TeX-master)))
+      (setq preview-auto-mode nil)
+      (user-error "Not in a LaTeX buffer, and TeX-master not set to a file"))
+    (unless TeX-header-end
+      (setq TeX-header-end LaTeX-header-end))
+    (unless TeX-trailer-start
+      (setq TeX-trailer-start LaTeX-trailer-start))
+    (unless comment-start-skip ; for texmathp
+      (setq comment-start-skip
+            (concat "\\(\\(^\\|[^\\\n]\\)\\("
+                    (regexp-quote TeX-esc)
+                    (regexp-quote TeX-esc)
+                    "\\)*\\)\\(%+[ \t]*\\)")))
+    (unless (memq #'preview-move-point post-command-hook)
+      (add-hook 'post-command-hook #'preview-move-point nil t))
+    (add-hook 'after-change-functions #'preview-auto--after-change nil t)
+    (add-hook 'post-command-hook #'preview-auto--post-command nil t)
+    (when preview-auto--timer
+      ;; Reset the timer, in case it's broken.
+      (cancel-timer preview-auto--timer)
+      (setq preview-auto--timer nil))
+    (setq preview-auto--timer (run-with-timer preview-auto-interval
+                                              preview-auto-interval
+                                              #'preview-auto--timer-function))
+    (setq preview-auto--keepalive t))
    (t
     (remove-hook 'after-change-functions #'preview-auto--after-change t)
     (remove-hook 'post-command-hook #'preview-auto--post-command t))))
 
+;;;###autoload
 (defun preview-auto-setup ()
-  "Hook function for installing bind and menu item."
+  "Set up keybinding and menu item for `preview-auto-mode'.
+This should be run after AUCTeX has loaded to a sufficient extent,
+e.g., in `LaTeX-mode-hook'."
   (remove-hook 'LaTeX-mode-hook #'preview-auto-setup)
   (define-key LaTeX-mode-map (kbd "C-c C-p C-a") #'preview-auto-mode)
   (easy-menu-add-item
@@ -550,7 +565,5 @@ cancel the preview, so that the preview is not misplaced."
    ["automatically" preview-auto-mode]
    "(or toggle) at point"))
 
-(add-hook 'LaTeX-mode-hook #'preview-auto-setup)
-
 (provide 'preview-auto)
 ;;; preview-auto.el ends here
